@@ -37,7 +37,7 @@ class Requester(object):
 
     def __getstate__(self):
         d = copy(self.__dict__)
-        del d['_session']
+        del d["_session"]
         return d
 
     def request(self, method, url, input_=None):
