@@ -14,6 +14,9 @@ from .video import Video
 
 from tusclient.uploader import Uploader
 
+from ..exceptions import InputError
+from ..infra.requester import Requester
+
 
 class Clock(TeamTVObject):
     @property
@@ -88,6 +91,12 @@ class Clock(TeamTVObject):
 
 
 class SportingEvent(TeamTVObject):
+    def __new__(cls, requester: Requester, attributes: dict):
+        if attributes["type"] == "match":
+            return super().__new__(MatchSportingEvent)
+        else:
+            return super().__new__(cls)
+
     @property
     def scheduled_at(self) -> datetime:
         return self._scheduled_at
@@ -105,8 +114,8 @@ class SportingEvent(TeamTVObject):
         return self._tags
 
     @property
-    def line_up_id(self):
-        return self._line_up_id
+    def type(self):
+        return self._type
 
     @property
     def main_video_id(self) -> Optional[str]:
@@ -115,16 +124,15 @@ class SportingEvent(TeamTVObject):
         return None
 
     @property
-    def is_local(self):
-        return self._is_local
-
-    @property
-    def home_team_id(self):
-        return self._home_team_id
-
-    @property
-    def away_team_id(self):
-        return self._away_team_id
+    def original(self) -> Optional["SportingEvent"]:
+        original_sporting_event_id = self.tags.get("copyOf")
+        if original_sporting_event_id:
+            return SportingEvent(
+                self._requester,
+                self._requester.request(
+                    "GET", f"/sportingEvents/{original_sporting_event_id}"
+                ),
+            )
 
     def get_clock(self, id_: str) -> Optional[Clock]:
         if id_ not in self._clocks:
@@ -147,7 +155,7 @@ class SportingEvent(TeamTVObject):
             if not video_id:
                 video_id = self.main_video_id
 
-            if video_id:
+            if video_id and video_id in self._clocks:
                 clock_id = self._clocks[video_id]["clockId"]
             else:
                 clock_id = "U1"
@@ -161,15 +169,6 @@ class SportingEvent(TeamTVObject):
             self,
         )
 
-    def get_team(self, team_id: str) -> dict:
-        home_team_name, away_team_name = self.name.split(" - ")
-        if team_id == self.home_team_id:
-            return {"name": home_team_name, "teamId": team_id, "ground": "home"}
-        elif team_id == self.away_team_id:
-            return {"name": away_team_name, "teamId": team_id, "ground": "away"}
-        else:
-            return {"name": None, "teamId": None, "ground": None}
-
     def add_bulk_observation(
         self, observations: List[DictObservation], description: Optional[str] = None
     ):
@@ -244,43 +243,81 @@ class SportingEvent(TeamTVObject):
             ),
         )
 
-    def get_line_up(self) -> LineUp:
-        data = self._requester.request("GET", f"/lineUps/{self.line_up_id}")
-        data["sportingEvent"] = self
-        return LineUp(self._requester, data)
-
     def upload_video(
         self,
         *file_paths: str,
         chunks_per_request: int = 1,
         tags: Optional[dict] = None,
         description: Optional[str] = None,
+        skip_transcoding: Optional[bool] = False,
+        resume_if_exists: Optional[bool] = False,
     ) -> Video:
         chunks_per_request = int(chunks_per_request)
+
         if chunks_per_request < 1:
             chunks_per_request = 1
 
-        files = []
-        for filename in file_paths:
-            files.append(
-                {"name": os.path.basename(filename), "size": os.path.getsize(filename)}
+        video = None
+        if resume_if_exists:
+            if not tags:
+                raise InputError(
+                    "When `resume_if_exists` is specified you must pass `tags`"
+                )
+
+            videos = self.get_videos_by_tags(**tags)
+            if not videos:
+                video = None
+            else:
+                # We found an existing video with matching tags. Check the files
+                video = videos[0]
+                if len(video.parts) != len(file_paths):
+                    raise InputError(
+                        f"Number of parts doesn't match. You specified {len(file_paths)} files. "
+                        f"The existing video ({video.video_id}) has {len(video.parts)} parts"
+                    )
+
+                for i, filename in enumerate(file_paths):
+                    if os.path.getsize(filename) != video.parts[i]["fileSize"]:
+                        raise InputError(
+                            f"File size of '{filename}' doesn't match existing video ({video.video_id}). "
+                            f"Local size: {os.path.getsize(filename)}. Remote size: {video.parts[i]['fileSize']}"
+                        )
+
+                if video.is_upload_completed:
+                    return video
+
+                video_id = video.video_id
+
+        if not video:
+            files = []
+            for filename in file_paths:
+                files.append(
+                    {
+                        "name": os.path.basename(filename),
+                        "size": os.path.getsize(filename),
+                    }
+                )
+
+            video_id = self._requester.request(
+                "POST",
+                f"/sportingEvents/{self.sporting_event_id}/initVideoUpload",
+                {
+                    "files": files,
+                    "tags": tags or {},
+                    "description": description,
+                    "skipTranscoding": skip_transcoding,
+                },
             )
 
-        video_id = self._requester.request(
-            "POST",
-            f"/sportingEvents/{self.sporting_event_id}/initVideoUpload",
-            {"files": files, "tags": tags or {}, "description": description},
-        )
+            video = Video(
+                self._requester, self._requester.request("GET", f"/videos/{video_id}")
+            )
 
-        video = Video(
-            self._requester, self._requester.request("GET", f"/videos/{video_id}")
-        )
         for i, part in enumerate(video.parts):
             uploader = Uploader(
                 file_paths[i],
                 url=part["tusUploadUrl"],
                 chunk_size=chunks_per_request * 25 * 1024 * 1024,
-                log_func=lambda msg: logging.info(f"Uploading: {msg}"),
                 retries=5,
                 retry_delay=5,
             )
@@ -294,18 +331,49 @@ class SportingEvent(TeamTVObject):
     def _use_attributes(self, attributes: dict):
         self._name = attributes["name"]
         self._sporting_event_id = attributes["sportingEventId"]
-        self._tags = attributes.get("tags", {})
+        self._tags = attributes.get("tags") or {}
         self._video_ids = attributes.get("videoIds", [])
         self._clocks = attributes["clocks"]
-        self._line_up_id = attributes["lineUpId"]
         self._scheduled_at = datetime.fromisoformat(
             attributes["scheduledAt"].replace("Z", "+00:00")
         )
-        self._is_local = attributes["_metadata"]["source"]["type"] == "ResourceGroup"
-        self._home_team_id = attributes["homeTeamId"]
-        self._away_team_id = attributes["awayTeamId"]
+        self._type = attributes["type"]
 
         super()._use_attributes(attributes)
 
     def __str__(self):
         return f"{self._scheduled_at.strftime('%d/%m/%y')} {self._name}"
+
+
+class MatchSportingEvent(SportingEvent):
+    @property
+    def home_team_id(self):
+        return self._home_team_id
+
+    @property
+    def away_team_id(self):
+        return self._away_team_id
+
+    @property
+    def line_up_id(self):
+        return self._line_up_id
+
+    def _use_attributes(self, attributes: dict):
+        self._line_up_id = attributes["lineUpId"]
+        self._home_team_id = attributes["homeTeamId"]
+        self._away_team_id = attributes["awayTeamId"]
+
+        super()._use_attributes(attributes)
+
+    def get_ground(self, team_id: str) -> Optional[str]:
+        if team_id == self.home_team_id:
+            return "home"
+        elif team_id == self.away_team_id:
+            return "away"
+        else:
+            return None
+
+    def get_line_up(self) -> LineUp:
+        data = self._requester.request("GET", f"/lineUps/{self.line_up_id}")
+        data["sportingEvent"] = self
+        return LineUp(self._requester, data)
